%% Copyright (C) 2023 Hongsheng Qi
%%
%% This program is free software: you can redistribute it and/or modify
%% it under the terms of the GNU General Public License as published by
%% the Free Software Foundation, either version 3 of the License, or
%% (at your option) any later version.
%%
%% This program is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with this program.  If not, see <https://www.gnu.org/licenses/>.

%% -*- texinfo -*-
%% @deftypefn {} {@var{retval} =} dksp (@var{A}, @var{B}, @var{...})
%%
%% @seealso{}
%% @end deftypefn

%% $Id: dksp.m 37 2023-06-12 09:55:12Z qihs $

function retval = dksp (A, B, varargin)

% DKSP    Dimension-Keeping (Left) Semi-Tensor Product of Matrices
	
if ~(isa(A,'sym') || isa(A,'double'))
	A = double(A);
end
if ~(isa(B,'sym') || isa(B,'double'))
	B = double(B);
end

if (ndims(A) > 2 || ndims(B) > 2)
	error('spdk: the input arguments must be 2-D.');
end

n = size(A,2);
p = size(B,1);
if n == p
    retval = A*B;
else
    t = lcm(n,p);
	retval = kron(A, ones(1,t/n)) * kron(B, ones(t/p,1));
end;

if length(varargin) > 0
	for i = 1:length(varargin)
		retval = dksp(retval, varargin{i});
	end
end
	
