%% Copyright (C) 2023 Hongsheng Qi
%%
%% This program is free software: you can redistribute it and/or modify
%% it under the terms of the GNU General Public License as published by
%% the Free Software Foundation, either version 3 of the License, or
%% (at your option) any later version.
%%
%% This program is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with this program.  If not, see <https://www.gnu.org/licenses/>.

%% -*- texinfo -*-
%% @deftypefn {} {@var{retval} =} mvsp (@var{A}, @var{v})
%%
%% @seealso{}
%% @end deftypefn

%% Author: Hongsheng Qi <qihongsh@amss.ac.cn>
%% Created: 2023-05-18

function retval = mvsp (A, v)
	
if ~(isa(A,'sym') || isa(A,'double'))
	A = double(A);
end
if ~(isa(v,'sym') || isa(v,'double'))
	v = double(v);
end

if (ndims(A) > 2)
	error('mvsp: the first input argument must be 2-D.');
end
if (ndims(v) > 2 || size(v,2) > 1)
	error('mvsp: the second input argument must be a column vector.');
end

n = size(A,2); % get the number of columns
p = size(v,1); % get the number of rows
if n == p
    retval = A*v;
else
    t = lcm(n,p);
    retval = kron(A,eye(t/n))*kron(v,ones(t/p,1));
end;
	
