function retval = sp(A,B,varargin)
% SP    (Left) Semi-Tensor Product of Matrices using Kronecker product
%
%   SP(A,B,...) is to calculate the (left) semi-tensor product of two or more arbitrary matrices.

% $Id: sp.m 36 2023-06-12 09:52:06Z qihs $

if nargin < 2
	error('Not enough input arguments.');
end

if ~(isa(A,'sym') || isa(A,'double'))
    A = double(A);
end
if ~(isa(B,'sym') || isa(B,'double'))
    B = double(B);
end

if (ndims(A) > 2) || (ndims(B) > 2)
    error('Input arguments must be 2-D.');
end

n = size(A,2); % get the number of columns
p = size(B,1); % get the number of rows
if n == p
    retval = A*B;
else
    t = lcm(n,p);
    retval = kron(A,eye(t/n))*kron(B,eye(t/p));
end;

if length(varargin) > 0
	for i = 1:length(varargin)
		retval = sp(retval, varargin{i});
	end
end