%% Copyright (C) 2023 Hongsheng Qi
%%
%% This program is free software: you can redistribute it and/or modify
%% it under the terms of the GNU General Public License as published by
%% the Free Software Foundation, either version 3 of the License, or
%% (at your option) any later version.
%%
%% This program is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with this program.  If not, see <https://www.gnu.org/licenses/>.

%% -*- texinfo -*-
%% @deftypefn {} {@var{retval} =} vvsp (@var{v1}, @var{v2})
%%
%% @seealso{}
%% @end deftypefn

%% Author: Hongsheng Qi <qihongsh@amss.ac.cn>
%% Created: 2023-05-18

function retval = vvsp (v1, v2)
	
if ~(isa(v1,'sym') || isa(v1,'double'))
	v1 = double(v1);
end
if ~(isa(v2,'sym') || isa(v2,'double'))
	v2 = double(v2);
end

[m1, n1] = size(v1);
[m2, n2] = size(v2);

if (ndims(v1) > 2 || (m1 > 1 && n1 > 1))
	error('vvsp: the first input argument must be a vector.');
end
if (ndims(v2) > 2 || (m2 > 1 && n2 > 1))
	error('vvsp: the second input argument must be a vector.');
end

if m1 > 1
	v1 = v1'; % convert to a row vector
	n1 = m1;
end
if n2 > 1
	v2 = v2'; % convert to a column vector
	m2 = n2;
end
% n1 = size(v1,2); % get the column number of v1
% m2 = size(v2,1); % get the row number of v2
if n1 == m2
    retval = v1*v2;
else
    t = lcm(n1,m2);
    retval = kron(v1,ones(1,t/n1))*kron(v2,ones(t/m2,1));
end;
